@@ -18,6 +18,7 @@ import string
 import numpy as np
 import time
 import collections
+import warnings
 import configparser
 import traceback
 import signal
@@ -64,6 +65,237 @@ class MissingLikelihoodError(Exception):
         self.pipeline_data = data
 
 
+class LimitedSizeDict(collections.OrderedDict):
+    "From http://stackoverflow.com/questions/2437617/limiting-the-size-of-a-python-dictionary"
+    def __init__(self, *args, **kwds):
+        self.size_limit = kwds.pop("size_limit", None)
+        collections.OrderedDict.__init__(self, *args, **kwds)
+        self._check_size_limit()
+
+    def __setitem__(self, key, value):
+        collections.OrderedDict.__setitem__(self, key, value)
+        self._check_size_limit()
+
+    def _check_size_limit(self):
+        if self.size_limit is not None:
+            while len(self) > self.size_limit:
+                self.popitem(last=False)
+
+
+class SlowSubspaceCache(object):
+    """
+    This tool analyzes pipelines to determine which of their parameters
+    are fast and which are slow, and then caches the results of new sets
+    of slow parameters so that if only fast parameters have changed the 
+    pipeline can be much faster.
+    """
+    def __init__(self, first_fast_module=None, cache_size_limit=3,):
+        self.current_hash = np.nan
+        self.analyzed = False
+        self.cache = LimitedSizeDict(size_limit=cache_size_limit)
+        self.first_fast_module = first_fast_module
+
+    def clear_cache(self):
+        self.cache.clear()
+        self.current_hash = np.nan
+
+    def hash_slow_parameters(self, block):
+        """This is not a general block hash! 
+        It just looks at the slow parameters.
+        """
+        pairs = block.keys()
+        pairs.sort()
+        values = []
+        for section,name in pairs:
+            if (section,name) not in self.slow_params:
+                continue
+            value = block[section, name]
+            if not np.isscalar(value):
+                #non-scalar in the block - this must
+                #reflect some feature not coded yet
+                warnings.warn("Vector parameter(s) in the input means that fast-slow split will not work - this can be fixed - please open an issue if it affects you.  Or set fast_slow=F")
+                return np.nan # 
+            values.append(value)
+        values = tuple(values)
+        return hash(values)
+
+    def start_pipeline(self, initial_block):
+        # We may be in the process of analyzing the pipeline
+        # the first time.
+        if not self.analyzed:
+            return 0
+        #
+        new_hash = self.hash_slow_parameters(initial_block)
+        cached = self.cache.get(new_hash)
+        if cached is None:
+            self.current_hash = new_hash
+            first_module = 0
+        else:            
+            # Now we need to use the old cached results in the new block.
+            # We put everything from the old block into the new block,
+            # EXCEPT for the fast parameters themselves.
+            for section,name in cached.keys():
+                if (section,name) not in self.fast_params:
+                    initial_block[section,name] = cached[section,name]
+            first_module = self.split_index
+        return first_module
+
+    def next_module_results(self, module_index, block):
+        if not self.analyzed:
+            return
+
+        if module_index != self.split_index-1:
+            return
+
+        self.cache[self.current_hash] = block.clone()
+
+    def analyze_pipeline(self, pipeline, all_params=False, grid=False):
+        """
+        Analyze the pipeline to determine how best to split it into two parts,
+        a slow beginning and a faster end, so that we can vary parameters
+
+        """
+
+        #Run the sampler twice first, once to make sure
+        #everything is initialized and once to do timing.
+        #Use the pipeline starting parameter since that is
+        #likely more typical than any random starting position
+        start = pipeline.start_vector()
+
+        if pipeline.has_run:
+            print("Pipeline has been run once already so no further initialization steps")
+        else:
+            print("")
+            print("Analyzing pipeline to determine fast and slow parameters (because fast_slow=T in [pipeline] section)")
+            print("")
+            print("Running pipeline once to make sure everything is initialized before timing.")
+            print("")
+            pipeline.posterior(start)
+        print("")
+        print("Running the pipeline again to determine timings and fast-slow split")
+        print("")
+        #Run with timing but make sure to re-set back to the original setting
+        #of timing after it finishes
+        #This will also print out the timing, which is handy.
+        original_timing = pipeline.timing
+        pipeline.timing = True
+        #Also get the datablock since it contains a log
+        #of all the parameter accesses
+        _, _, block = pipeline.posterior(start, return_data=True)
+        pipeline.timing = original_timing
+        timings = pipeline.timings
+
+        #Now we have the datablock, which has the log in it, and the timing.
+        #The only information that can be of relevance is the fraction
+        #of the time pipeline before a given module, and the list of 
+        #parameters accessed before that module.
+        #We are going to need to go through the log and figure out the latter of
+        #these
+        if all_params:
+            params = pipeline.parameters
+        else:
+            params = pipeline.varied_params
+        first_use = block.get_first_parameter_use(params)
+        first_use_count = [len(f) for f in first_use.values()]
+        if sum(first_use_count)!=len(params):
+            print(first_use)
+            print(params)
+            raise ValueError("Tried to do fast-slow split but not all varied parameters ever used in the pipeline (used {}, have{})".format(sum(first_use_count), len(params)))
+        print("\n")
+        print("Parameters first used in each module:")
+        for f, n in zip(first_use.items(), first_use_count):
+            name, params = f
+            print("{} - {} parameters:".format(name, n))
+            for p in params:
+                print("     {}--{}".format(*p))
+
+        # Now we have a count of the number of parameters and amount of 
+        # time used before each module in the pipeline
+        # So we can divide up the parameters into fast and slow.
+        self.split_index = self._choose_fast_slow_split(first_use_count, timings, grid)
+
+        full_time = sum(timings)
+        slow_time = sum(timings[:self.split_index])
+        fast_time = sum(timings[self.split_index:])
+
+        print("Time for full pipeline:  {:.2f}s".format(full_time))
+        print("Time for slow pipeline:  {:.2f}s".format(slow_time))
+        print("Time for fast pipeline:  {:.2f}s".format(fast_time))
+        time_save_percent = 100-100*fast_time/full_time
+        print("Time saving: {:.2f}%".format(time_save_percent))
+
+        worth_splitting = time_save_percent > 10.
+
+        if not worth_splitting:
+            print("")
+            print("No significant time saving (<10%) from a fast-slow split.")
+            print("Not splitting pipeline into fast and slow parts.")
+            print("")
+            self.split_index = len(timings)
+
+        self.slow_modules = self.split_index
+        self.fast_modules = len(pipeline.modules) - self.slow_modules
+        self.slow_params = sum(list(first_use.values())[:self.split_index], [])
+        self.fast_params = sum(list(first_use.values())[self.split_index:], [])
+
+        if worth_splitting:
+            print("")
+            print("Based on this we have decided: ")
+            print("   Slow modules ({}):".format(self.slow_modules))
+            for module in pipeline.modules[:self.split_index]:
+                print("        %s"% module.name)
+            print("   Fast modules ({}):".format(self.fast_modules))
+            for module in pipeline.modules[self.split_index:]:
+                print("        {}".format(module.name))
+            print("   Slow parameters ({}):".format(len(self.slow_params)))
+            for param in self.slow_params:
+                print("        {}--{}".format(*param))
+            print("   Fast parameters ({}):".format(len(self.fast_params)))
+            for param in self.fast_params:
+                print("        {}--{}".format(*param))
+            print("")
+            print("")
+        self.analyzed = True
+
+    def _choose_fast_slow_split(self, first_use_count, timings, grid_mode):
+        #some kind of algorithm to loop through the modules
+        #and work out the time saving if we did the fast/slow from there.
+        #at the moment just return the last module where there are any 
+        #parameters
+        #n = len(slow_count)
+        if self.first_fast_module is not None:
+            print("You manually told me in the parameter file to use module {} as the first in the fast block".format(self.first_fast_module))
+            return self.first_fast_module
+
+
+        n_step = len(timings)
+        n_total = sum(first_use_count)
+        T_total = sum(timings)
+
+        if grid_mode:
+            print("Analyzing the fast/slow for grid sampler - different rule used for time saving")
+
+        T = np.zeros(n_step)
+        for i in range(n_step):
+            T_slow = sum(timings[:i])
+            n_slow = sum(first_use_count[:i])
+            T_fast = T_total - T_slow
+            n_fast = n_total - n_slow
+            if n_fast==0:
+                T[i] = T_total
+            elif grid_mode:
+                T[i] = (T_slow/T_total)*10**(-n_fast) + (T_fast/T_total)*(1-10**(-n_fast))
+            else:
+                T[i] = (T_fast*n_fast)/(T_total*n_total) + (T_slow*n_slow)/(n_total*T_total)
+
+        index = T.argmin()
+        Tmin = T.min()
+        index = (np.where(T==Tmin)[0]).max()
+        print("Will use module {} as the first in the fast block".format(index))
+
+        return index
+
+
 
 class Pipeline(object):
 
@@ -115,23 +347,34 @@ class Pipeline(object):
 
         base_directory = self.base_directory()
 
+
         self.quiet = self.options.getboolean(PIPELINE_INI_SECTION, "quiet", fallback=True)
         self.debug = self.options.getboolean(PIPELINE_INI_SECTION, "debug", fallback=False)
         self.timing = self.options.getboolean(PIPELINE_INI_SECTION, "timing", fallback=False)
         shortcut = self.options.get(PIPELINE_INI_SECTION, "shortcut", fallback="")
-        if shortcut=="": shortcut=None
+        if shortcut=="":
+            shortcut=None
+
+        self.do_fast_slow = self.options.getboolean(PIPELINE_INI_SECTION, "fast_slow", fallback=False)
+        if self.do_fast_slow and shortcut:
+            sys.stderr.write("Warning: you have the fast_slow and shortcut options both set, and we can only do one of those at once (we will do shortcut)\n")
+            self.do_fast_slow = False
+        self.slow_subspace_cache = None #until set in method
+        self.first_fast_module = self.options.get(PIPELINE_INI_SECTION, "first_fast_module", fallback="")
 
         # initialize modules
         self.modules = []
+        self.has_run = False
+
         if load and PIPELINE_INI_SECTION in self.options.sections():
             module_list = self.options.get(PIPELINE_INI_SECTION,
                                            "modules", fallback="").split()
-
             self.modules = [
                 module.Module.from_options(module_name,self.options,base_directory)
                 for module_name in module_list
             ]
 
+
             self.shortcut_module=0
             self.shortcut_data=None
             if shortcut is not None:
@@ -205,6 +448,41 @@ class Pipeline(object):
                 sys.stdout.write("%s %f\n" % (name, t2-t1))
 
 
+    def setup_fast_subspaces(self, all_params=False, grid=False):
+        if self.do_fast_slow:
+            print("Doing fast/slow parameter splitting")
+            if self.first_fast_module:
+                for i,module in enumerate(self.modules):
+                    if module.name==self.first_fast_module:
+                        first_fast_index=i
+                        break
+                else:
+                    raise ValueError("You set first_fast_module={} but never ran that module".format(self.first_fast_module))
+            else:
+                first_fast_index = None
+
+            self.slow_subspace_cache = SlowSubspaceCache(first_fast_module=first_fast_index)
+            self.slow_subspace_cache.analyze_pipeline(self, all_params=all_params, grid=grid)
+            # This looks a bit weird but makes sure that self.fast_params
+            # and self.slow_params contain objects of type Parameter
+            # not just the (section,name) tuples.
+            if all_params:
+                params = self.parameters
+            else:
+                params = self.varied_params
+            self.fast_params = [p for p in params 
+                if p in self.slow_subspace_cache.fast_params]
+            self.slow_params = [p for p in params 
+                if p in self.slow_subspace_cache.slow_params]
+            self.fast_param_indices = [params.index(p)
+                for p in self.fast_params]
+            self.slow_param_indices = [params.index(p)
+                for p in self.slow_params]
+            self.n_fast_params = len(self.fast_params)
+            self.n_slow_params = len(self.slow_params)
+        else:
+            self.slow_subspace_cache = None
+
 
     def cleanup(self):
         u"""Call every `module`ʼs `cleanup` method."""
@@ -293,13 +571,27 @@ class Pipeline(object):
 
         """
         modules = self.modules
-        first = (self.shortcut_data is None)
+
+        timings = []
+        if self.shortcut_module:
+            if self.shortcut_data is None:
+                first_module = 0
+            else:
+                first_module = shortcut_module_index
+        elif self.slow_subspace_cache:
+            first_module = self.slow_subspace_cache.start_pipeline(data_package)
+            if first_module != 0 and (self.debug or self.timing):
+                sys.stdout.write("COOL: Quickstarting pipeline from module {} (fast/slow)\n".format(first_module))
+                sys.stdout.flush()
+        else:
+            first_module = 0
+
         if self.timing:
             start_time = time.time()
-        if self.shortcut_module and not first:
-            modules = modules[self.shortcut_module:]
 
         for module_number, module in enumerate(modules):
+            if module_number<first_module:
+                continue
             if self.debug:
                 sys.stdout.write("Running %.20s ...\n" % module)
                 sys.stdout.flush()
@@ -320,6 +612,7 @@ class Pipeline(object):
 
             if self.timing:
                 t2 = time.time()
+                timings.append(t2-t1)
                 sys.stdout.write("%s took: %.3f seconds\n"% (module,t2-t1))
 
             if status:
@@ -339,22 +632,33 @@ class Pipeline(object):
                         sys.stderr.write("Setting debug=T in [pipeline] might help.\n")
                 return None
 
-            if self.shortcut_module and first and module_number==self.shortcut_module-1:
+            # If we are using a fast/slow split (and we are not already running on a cached subset)
+            # Then see if it wants to cache these results
+            if self.slow_subspace_cache and first_module==0:
+                self.slow_subspace_cache.next_module_results(module_number, data_package)
+
+            # Alternatively we will do the shortcut thing
+            elif self.shortcut_module and not self.has_run and module_number==self.shortcut_module-1:
                 print("Saving shortcut data")
                 self.shortcut_data = data_package.clone()
 
         if self.timing:
             end_time = time.time()
             sys.stdout.write("Total pipeline time: {:.3} seconds\n".format(end_time-start_time))
-
+            self.timings = timings
 
         if not self.quiet:
             sys.stdout.write("Pipeline ran okay.\n")
 
         data_package.log_access("MODULE-START", "Results", "")
         # return something
+        self.has_run = True
         return True
 
+    def clear_cache(self):
+        self.slow_subspace_cache.clear_cache()
+
+
 
 
 class LikelihoodPipeline(Pipeline):
@@ -385,7 +689,7 @@ class LikelihoodPipeline(Pipeline):
 
     """
 
-    def __init__(self, arg=None, id="",override=None, load=True):
+    def __init__(self, arg=None, id="",override=None, load=True, values=None, priors=None):
         u"""Construct a :class:`LikelihoodPipeline`.
 
         The arguments `arg` and `load` are used in the base-class
@@ -403,17 +707,24 @@ class LikelihoodPipeline(Pipeline):
             self.id_code = ""
         self.n_iterations = 0
 
-        values_file = self.options.get(PIPELINE_INI_SECTION, "values")
-        self.values_filename=values_file
-        priors_files = self.options.get(PIPELINE_INI_SECTION,
-                                        "priors", fallback="").split()
-        self.priors_files = priors_files
 
-        self.parameters = parameter.Parameter.load_parameters(values_file,
-                                                              priors_files,
+        if values is None:
+            self.values_file = self.options.get(PIPELINE_INI_SECTION, "values")
+            self.values_filename = self.values_file
+        else:
+            self.values_file = values
+            self.values_filename = None
+
+        if priors is None:
+            priors_files = self.options.get(PIPELINE_INI_SECTION,
+                                            "priors", fallback="").split()
+            self.priors_files = priors_files
+        else:
+            self.priors_files = priors
+        self.parameters = parameter.Parameter.load_parameters(self.values_file,
+                                                              self.priors_files,
                                                               override,
                                                               )
-
         self.reset_fixed_varied_parameters()
 
         self.print_priors()
@@ -431,7 +742,8 @@ class LikelihoodPipeline(Pipeline):
         #pull out all the section names and likelihood names for later
 
         likelihood_names = self.options.get(PIPELINE_INI_SECTION,
-                                            "likelihoods", NO_LIKELIHOOD_NAMES)
+                                            "likelihoods",
+                                            fallback=NO_LIKELIHOOD_NAMES)
         if likelihood_names==NO_LIKELIHOOD_NAMES:
             self.likelihood_names = NO_LIKELIHOOD_NAMES
         else:
@@ -462,6 +774,8 @@ class LikelihoodPipeline(Pipeline):
                               if param.is_varied()]
         self.fixed_params = [param for param in self.parameters
                              if param.is_fixed()]
+        self.nvaried = len(self.varied_params)
+        self.nfixed = len(self.fixed_params)
 
 
 
@@ -657,11 +971,10 @@ class LikelihoodPipeline(Pipeline):
             return np.array([param.limits[1] for
                          param in self.varied_params])
 
+    def build_starting_block(self, p, check_ranges=False, all_params=False):
+        u"""Assemble :class:`DataBlock` data based on parameter values in `p`, and return it.
 
 
-    def run_parameters(self, p, check_ranges=False, all_params=False):
-        u"""Assemble :class:`DataBlock` data based on parameter values in `p`, and run the pipeline on those data.
-
         If `check_ranges` is indicated, the function will return `None` if
         **any** of our parameters are out of their indicated range.
 
@@ -675,10 +988,7 @@ class LikelihoodPipeline(Pipeline):
             if self.is_out_of_range(p):
                 return None
 
-        if self.shortcut_module and self.shortcut_data is not None:
-            data = self.shortcut_data.clone()
-        else:
-            data = block.DataBlock()
+        data = block.DataBlock()
 
         if all_params:
             for param, x in zip(self.parameters, p):
@@ -692,6 +1002,23 @@ class LikelihoodPipeline(Pipeline):
             for param in self.fixed_params:
                 data[param.section, param.name] = param.start
 
+        return data
+
+
+    def run_parameters(self, p, check_ranges=False, all_params=False):
+        u"""Assemble :class:`DataBlock` data based on parameter values in `p`, and run the pipeline on those data.
+
+        If `check_ranges` is indicated, the function will return `None` if
+        **any** of our parameters are out of their indicated range.
+
+        If `all_params` is indicated, then the `p` run data will be
+        assumed to match all the pipeline parameter, including fixed ones.
+        Otherwise (the default) it should match the list ‘varied_params’, and all
+        of our ‘fixed’ parameters are added to the run-set.
+
+        """
+        data = self.build_starting_block(p, check_ranges=check_ranges, all_params=all_params)
+
         if self.run(data):
             return data
         else:
@@ -832,7 +1159,7 @@ class LikelihoodPipeline(Pipeline):
             self._set_likelihood_names_from_block(data)
 
             if not self.quiet:
-                # Tell the suer what we found
+                # Tell the user what we found.
                 print("Likelihoods not set in parameter file, so checking what is generated:")
                 for name in self.likelihood_names:
                     print("Found likelihood named {}".format(name))
@@ -869,10 +1196,9 @@ class LikelihoodPipeline(Pipeline):
         u"""Run the simulation pipeline, computing any log-likelihoods in the pipeline 
         given the given input parameter values, and return the sum of these.
 
-        The parameter vector `p` must match the length of `self.varied_params`, unless
-        `all_params` is specified as `True` in which case it must match
-        `self.parameters', i.e. must correspond to the complete parameter
-        set.
+        The parameter vector `p` must match the length of `self.varied_params`,
+        unless `all_params` is specified as `True` in which case it must match
+        `self.parameters', i.e. must correspond to the complete parameter set.
 
         If `return_data` are requested, then the updated data block will
         be returned as the third return item.
